# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class SummarizeRequestExtractiveness(str, enum.Enum):
    """
    One of `low`, `medium`, `high`, or `auto`, defaults to `auto`. Controls how close to the original text the summary is. `high` extractiveness summaries will lean towards reusing sentences verbatim, while `low` extractiveness summaries will tend to paraphrase more. If `auto` is selected, the best option will be picked based on the input text.
    """

    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"

    def visit(
        self,
        low: typing.Callable[[], T_Result],
        medium: typing.Callable[[], T_Result],
        high: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is SummarizeRequestExtractiveness.LOW:
            return low()
        if self is SummarizeRequestExtractiveness.MEDIUM:
            return medium()
        if self is SummarizeRequestExtractiveness.HIGH:
            return high()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .api_meta import ApiMeta

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class EmbedFloatsResponse(pydantic.BaseModel):
    id: str
    embeddings: typing.List[typing.List[float]] = pydantic.Field(
        description="An array of embeddings, where each embedding is an array of floats. The length of the `embeddings` array will be the same as the length of the original `texts` array."
    )
    texts: typing.List[str] = pydantic.Field(description="The text entries for which embeddings were returned.")
    meta: typing.Optional[ApiMeta] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ConnectorAuthStatus(str, enum.Enum):
    """
    The OAuth status for the user making the request. One of ["valid", "expired", ""]. Empty string (field is omitted) means the user has not authorized the connector yet.
    """

    VALID = "valid"
    EXPIRED = "expired"

    def visit(self, valid: typing.Callable[[], T_Result], expired: typing.Callable[[], T_Result]) -> T_Result:
        if self is ConnectorAuthStatus.VALID:
            return valid()
        if self is ConnectorAuthStatus.EXPIRED:
            return expired()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AuthTokenType(str, enum.Enum):
    """
    The token_type specifies the way the token is passed in the Authorization header. Valid values are "bearer", "basic", and "noscheme".
    """

    BEARER = "bearer"
    BASIC = "basic"
    NOSCHEME = "noscheme"

    def visit(
        self,
        bearer: typing.Callable[[], T_Result],
        basic: typing.Callable[[], T_Result],
        noscheme: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AuthTokenType.BEARER:
            return bearer()
        if self is AuthTokenType.BASIC:
            return basic()
        if self is AuthTokenType.NOSCHEME:
            return noscheme()



# This file was auto-generated by Fern from our API Definition.

import typing

from .rerank_request_documents_item_text import RerankRequestDocumentsItemText

RerankRequestDocumentsItem = typing.Union[str, RerankRequestDocumentsItemText]



# This file was auto-generated by Fern from our API Definition.

from .api_error import ApiError
from .client_wrapper import AsyncClientWrapper, BaseClientWrapper, SyncClientWrapper
from .datetime_utils import serialize_datetime
from .file import File, convert_file_dict_to_httpx_tuples
from .jsonable_encoder import jsonable_encoder
from .remove_none_from_dict import remove_none_from_dict
from .request_options import RequestOptions

__all__ = [
    "ApiError",
    "AsyncClientWrapper",
    "BaseClientWrapper",
    "File",
    "RequestOptions",
    "SyncClientWrapper",
    "convert_file_dict_to_httpx_tuples",
    "jsonable_encoder",
    "remove_none_from_dict",
    "serialize_datetime",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .chat_citation import ChatCitation
from .chat_document import ChatDocument
from .chat_search_query import ChatSearchQuery
from .chat_search_result import ChatSearchResult
from .finish_reason import FinishReason

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class NonStreamedChatResponse(pydantic.BaseModel):
    text: str = pydantic.Field(description="Contents of the reply generated by the model.")
    generation_id: typing.Optional[str] = pydantic.Field(
        default=None, description="Unique identifier for the generated reply. Useful for submitting feedback."
    )
    citations: typing.Optional[typing.List[ChatCitation]] = pydantic.Field(
        default=None, description="Inline citations for the generated reply."
    )
    documents: typing.Optional[typing.List[ChatDocument]] = pydantic.Field(
        default=None, description="Documents seen by the model when generating the reply."
    )
    is_search_required: typing.Optional[bool] = pydantic.Field(
        default=None, description="Denotes that a search for documents is required during the RAG flow."
    )
    search_queries: typing.Optional[typing.List[ChatSearchQuery]] = pydantic.Field(
        default=None, description="Generated search queries, meant to be used as part of the RAG flow."
    )
    search_results: typing.Optional[typing.List[ChatSearchResult]] = pydantic.Field(
        default=None, description="Documents retrieved from each of the conducted searches."
    )
    finish_reason: typing.Optional[FinishReason] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class EmbedByTypeResponseEmbeddings(pydantic.BaseModel):
    """
    An object with different embedding types. The length of each embedding type array will be the same as the length of the original `texts` array.
    """

    float_: typing.Optional[typing.List[typing.List[float]]] = pydantic.Field(
        alias="float", default=None, description="An array of float embeddings."
    )
    int_8: typing.Optional[typing.List[typing.List[float]]] = pydantic.Field(
        alias="int8",
        default=None,
        description="An array of signed int8 embeddings. Each value is between -128 and 127.",
    )
    uint_8: typing.Optional[typing.List[typing.List[float]]] = pydantic.Field(
        alias="uint8",
        default=None,
        description="An array of unsigned int8 embeddings. Each value is between 0 and 255.",
    )
    binary: typing.Optional[typing.List[typing.List[float]]] = pydantic.Field(
        default=None,
        description="An array of packed signed binary embeddings. The length of each binary embedding is 1/8 the length of the float embeddings of the provided model. Each value is between -128 and 127.",
    )
    ubinary: typing.Optional[typing.List[typing.List[float]]] = pydantic.Field(
        default=None,
        description="An array of packed unsigned binary embeddings. The length of each binary embedding is 1/8 the length of the float embeddings of the provided model. Each value is between 0 and 255.",
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .classify_response_classifications_item_classification_type import (
    ClassifyResponseClassificationsItemClassificationType,
)
from .classify_response_classifications_item_labels_value import ClassifyResponseClassificationsItemLabelsValue

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClassifyResponseClassificationsItem(pydantic.BaseModel):
    id: str
    input: typing.Optional[str] = pydantic.Field(default=None, description="The input text that was classified")
    prediction: typing.Optional[str] = pydantic.Field(
        default=None, description="The predicted label for the associated query (only filled for single-label models)"
    )
    predictions: typing.List[str] = pydantic.Field(
        description="An array containing the predicted labels for the associated query (only filled for single-label classification)"
    )
    confidence: typing.Optional[float] = pydantic.Field(
        default=None,
        description="The confidence score for the top predicted class (only filled for single-label classification)",
    )
    confidences: typing.List[float] = pydantic.Field(
        description="An array containing the confidence scores of all the predictions in the same order"
    )
    labels: typing.Dict[str, ClassifyResponseClassificationsItemLabelsValue] = pydantic.Field(
        description="A map containing each label and its confidence score according to the classifier. All the confidence scores add up to 1 for single-label classification. For multi-label classification the label confidences are independent of each other, so they don't have to sum up to 1."
    )
    classification_type: ClassifyResponseClassificationsItemClassificationType = pydantic.Field(
        description="The type of classification performed"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .chat_citation_generation_event import ChatCitationGenerationEvent
from .chat_search_queries_generation_event import ChatSearchQueriesGenerationEvent
from .chat_search_results_event import ChatSearchResultsEvent
from .chat_stream_end_event import ChatStreamEndEvent
from .chat_stream_start_event import ChatStreamStartEvent
from .chat_text_generation_event import ChatTextGenerationEvent
from .chat_tool_inputs_generation_event import ChatToolInputsGenerationEvent


class StreamedChatResponse_StreamStart(ChatStreamStartEvent):
    event_type: typing.Literal["stream-start"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class StreamedChatResponse_SearchQueriesGeneration(ChatSearchQueriesGenerationEvent):
    event_type: typing.Literal["search-queries-generation"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class StreamedChatResponse_SearchResults(ChatSearchResultsEvent):
    event_type: typing.Literal["search-results"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class StreamedChatResponse_TextGeneration(ChatTextGenerationEvent):
    event_type: typing.Literal["text-generation"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class StreamedChatResponse_CitationGeneration(ChatCitationGenerationEvent):
    event_type: typing.Literal["citation-generation"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class StreamedChatResponse_ToolInputsGeneration(ChatToolInputsGenerationEvent):
    event_type: typing.Literal["tool-inputs-generation"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class StreamedChatResponse_StreamEnd(ChatStreamEndEvent):
    event_type: typing.Literal["stream-end"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


StreamedChatResponse = typing.Union[
    StreamedChatResponse_StreamStart,
    StreamedChatResponse_SearchQueriesGeneration,
    StreamedChatResponse_SearchResults,
    StreamedChatResponse_TextGeneration,
    StreamedChatResponse_CitationGeneration,
    StreamedChatResponse_ToolInputsGeneration,
    StreamedChatResponse_StreamEnd,
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class GenerateStreamRequestTruncate(str, enum.Enum):
    """
    One of `NONE|START|END` to specify how the API will handle inputs longer than the maximum token length.

    Passing `START` will discard the start of the input. `END` will discard the end of the input. In both cases, input is discarded until the remaining input is exactly the maximum input token length for the model.

    If `NONE` is selected, when the input exceeds the maximum input token length an error will be returned.
    """

    NONE = "NONE"
    START = "START"
    END = "END"

    def visit(
        self,
        none: typing.Callable[[], T_Result],
        start: typing.Callable[[], T_Result],
        end: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is GenerateStreamRequestTruncate.NONE:
            return none()
        if self is GenerateStreamRequestTruncate.START:
            return start()
        if self is GenerateStreamRequestTruncate.END:
            return end()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class SummarizeRequestLength(str, enum.Enum):
    """
    One of `short`, `medium`, `long`, or `auto` defaults to `auto`. Indicates the approximate length of the summary. If `auto` is selected, the best option will be picked based on the input text.
    """

    SHORT = "short"
    MEDIUM = "medium"
    LONG = "long"

    def visit(
        self,
        short: typing.Callable[[], T_Result],
        medium: typing.Callable[[], T_Result],
        long: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is SummarizeRequestLength.SHORT:
            return short()
        if self is SummarizeRequestLength.MEDIUM:
            return medium()
        if self is SummarizeRequestLength.LONG:
            return long()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ChatMessageRole(str, enum.Enum):
    """
    One of CHATBOT|USER to identify who the message is coming from.
    """

    CHATBOT = "CHATBOT"
    USER = "USER"

    def visit(self, chatbot: typing.Callable[[], T_Result], user: typing.Callable[[], T_Result]) -> T_Result:
        if self is ChatMessageRole.CHATBOT:
            return chatbot()
        if self is ChatMessageRole.USER:
            return user()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ChatStreamEndEventFinishReason(str, enum.Enum):
    """
    - `COMPLETE` - the model sent back a finished reply
    - `ERROR_LIMIT` - the reply was cut off because the model reached the maximum number of tokens for its context length
    - `MAX_TOKENS` - the reply was cut off because the model reached the maximum number of tokens specified by the max_tokens parameter
    - `ERROR` - something went wrong when generating the reply
    - `ERROR_TOXIC` - the model generated a reply that was deemed toxic
    """

    COMPLETE = "COMPLETE"
    ERROR_LIMIT = "ERROR_LIMIT"
    MAX_TOKENS = "MAX_TOKENS"
    ERROR = "ERROR"
    ERROR_TOXIC = "ERROR_TOXIC"

    def visit(
        self,
        complete: typing.Callable[[], T_Result],
        error_limit: typing.Callable[[], T_Result],
        max_tokens: typing.Callable[[], T_Result],
        error: typing.Callable[[], T_Result],
        error_toxic: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ChatStreamEndEventFinishReason.COMPLETE:
            return complete()
        if self is ChatStreamEndEventFinishReason.ERROR_LIMIT:
            return error_limit()
        if self is ChatStreamEndEventFinishReason.MAX_TOKENS:
            return max_tokens()
        if self is ChatStreamEndEventFinishReason.ERROR:
            return error()
        if self is ChatStreamEndEventFinishReason.ERROR_TOXIC:
            return error_toxic()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EmbedJobStatus(str, enum.Enum):
    """
    The status of the embed job
    """

    PROCESSING = "processing"
    COMPLETE = "complete"
    CANCELLING = "cancelling"
    CANCELLED = "cancelled"
    FAILED = "failed"

    def visit(
        self,
        processing: typing.Callable[[], T_Result],
        complete: typing.Callable[[], T_Result],
        cancelling: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is EmbedJobStatus.PROCESSING:
            return processing()
        if self is EmbedJobStatus.COMPLETE:
            return complete()
        if self is EmbedJobStatus.CANCELLING:
            return cancelling()
        if self is EmbedJobStatus.CANCELLED:
            return cancelled()
        if self is EmbedJobStatus.FAILED:
            return failed()



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .generate_stream_end import GenerateStreamEnd
from .generate_stream_error import GenerateStreamError
from .generate_stream_text import GenerateStreamText


class GenerateStreamedResponse_TextGeneration(GenerateStreamText):
    event_type: typing.Literal["text-generation"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class GenerateStreamedResponse_StreamEnd(GenerateStreamEnd):
    event_type: typing.Literal["stream-end"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class GenerateStreamedResponse_StreamError(GenerateStreamError):
    event_type: typing.Literal["stream-error"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


GenerateStreamedResponse = typing.Union[
    GenerateStreamedResponse_TextGeneration, GenerateStreamedResponse_StreamEnd, GenerateStreamedResponse_StreamError
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .connector_auth_status import ConnectorAuthStatus
from .connector_o_auth import ConnectorOAuth

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Connector(pydantic.BaseModel):
    """
    A connector allows you to integrate data sources with the '/chat' endpoint to create grounded generations with citations to the data source.
    documents to help answer users.
    """

    id: str = pydantic.Field(
        description=(
            "The unique identifier of the connector (used in both `/connectors` & `/chat` endpoints).\n"
            "This is automatically created from the name of the connector upon registration.\n"
        )
    )
    organization_id: typing.Optional[str] = pydantic.Field(
        default=None,
        description=(
            "The organization to which this connector belongs. This is automatically set to\n"
            "the organization of the user who created the connector.\n"
        ),
    )
    name: str = pydantic.Field(description="A human-readable name for the connector.")
    description: typing.Optional[str] = pydantic.Field(default=None, description="A description of the connector.")
    url: typing.Optional[str] = pydantic.Field(
        default=None, description="The URL of the connector that will be used to search for documents."
    )
    created_at: dt.datetime = pydantic.Field(description="The UTC time at which the connector was created.")
    updated_at: dt.datetime = pydantic.Field(description="The UTC time at which the connector was last updated.")
    excludes: typing.Optional[typing.List[str]] = pydantic.Field(
        default=None, description="A list of fields to exclude from the prompt (fields remain in the document)."
    )
    auth_type: typing.Optional[str] = pydantic.Field(
        default=None,
        description="The type of authentication/authorization used by the connector. Possible values: [oauth, service_auth]",
    )
    oauth: typing.Optional[ConnectorOAuth] = pydantic.Field(
        default=None, description="The OAuth 2.0 configuration for the connector."
    )
    auth_status: typing.Optional[ConnectorAuthStatus] = pydantic.Field(
        default=None,
        description='The OAuth status for the user making the request. One of ["valid", "expired", ""]. Empty string (field is omitted) means the user has not authorized the connector yet.',
    )
    active: typing.Optional[bool] = pydantic.Field(default=None, description="Whether the connector is active or not.")
    continue_on_failure: typing.Optional[bool] = pydantic.Field(
        default=None,
        description="Whether a chat request should continue or not if the request to this connector fails.",
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...types.create_embed_job_response import CreateEmbedJobResponse
from ...types.embed_input_type import EmbedInputType
from ...types.embed_job import EmbedJob
from ...types.list_embed_job_response import ListEmbedJobResponse
from .types.create_embed_job_request_truncate import CreateEmbedJobRequestTruncate

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EmbedJobsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListEmbedJobResponse:
        """
        The list embed job endpoint allows users to view all embed jobs history for that specific user.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.embed_jobs.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "embed-jobs"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListEmbedJobResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        model: str,
        dataset_id: str,
        input_type: EmbedInputType,
        name: typing.Optional[str] = OMIT,
        truncate: typing.Optional[CreateEmbedJobRequestTruncate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateEmbedJobResponse:
        """
        This API launches an async Embed job for a [Dataset](https://docs.cohere.com/docs/datasets) of type `embed-input`. The result of a completed embed job is new Dataset of type `embed-output`, which contains the original text entries and the corresponding embeddings.

        Parameters:
            - model: str. ID of the embedding model.

                          Available models and corresponding embedding dimensions:

                          - `embed-english-v3.0` : 1024
                          - `embed-multilingual-v3.0` : 1024
                          - `embed-english-light-v3.0` : 384
                          - `embed-multilingual-light-v3.0` : 384

            - dataset_id: str. ID of a [Dataset](https://docs.cohere.com/docs/datasets). The Dataset must be of type `embed-input` and must have a validation status `Validated`

            - input_type: EmbedInputType.

            - name: typing.Optional[str]. The name of the embed job.

            - truncate: typing.Optional[CreateEmbedJobRequestTruncate]. One of `START|END` to specify how the API will handle inputs longer than the maximum token length.

                                                                        Passing `START` will discard the start of the input. `END` will discard the end of the input. In both cases, input is discarded until the remaining input is exactly the maximum input token length for the model.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere import EmbedInputType
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.embed_jobs.create(
            model="model",
            dataset_id="dataset_id",
            input_type=EmbedInputType.SEARCH_DOCUMENT,
        )
        """
        _request: typing.Dict[str, typing.Any] = {"model": model, "dataset_id": dataset_id, "input_type": input_type}
        if name is not OMIT:
            _request["name"] = name
        if truncate is not OMIT:
            _request["truncate"] = truncate.value if truncate is not None else None
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "embed-jobs"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmbedJobResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmbedJob:
        """
        This API retrieves the details about an embed job started by the same user.

        Parameters:
            - id: str. The ID of the embed job to retrieve.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.embed_jobs.get(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"embed-jobs/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmbedJob, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        This API allows users to cancel an active embed job. Once invoked, the embedding process will be terminated, and users will be charged for the embeddings processed up to the cancellation point. It's important to note that partial results will not be available to users after cancellation.

        Parameters:
            - id: str. The ID of the embed job to cancel.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import BaseCohere

        client = BaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        client.embed_jobs.cancel(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"embed-jobs/{jsonable_encoder(id)}/cancel"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncEmbedJobsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListEmbedJobResponse:
        """
        The list embed job endpoint allows users to view all embed jobs history for that specific user.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.embed_jobs.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "embed-jobs"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListEmbedJobResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        model: str,
        dataset_id: str,
        input_type: EmbedInputType,
        name: typing.Optional[str] = OMIT,
        truncate: typing.Optional[CreateEmbedJobRequestTruncate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateEmbedJobResponse:
        """
        This API launches an async Embed job for a [Dataset](https://docs.cohere.com/docs/datasets) of type `embed-input`. The result of a completed embed job is new Dataset of type `embed-output`, which contains the original text entries and the corresponding embeddings.

        Parameters:
            - model: str. ID of the embedding model.

                          Available models and corresponding embedding dimensions:

                          - `embed-english-v3.0` : 1024
                          - `embed-multilingual-v3.0` : 1024
                          - `embed-english-light-v3.0` : 384
                          - `embed-multilingual-light-v3.0` : 384

            - dataset_id: str. ID of a [Dataset](https://docs.cohere.com/docs/datasets). The Dataset must be of type `embed-input` and must have a validation status `Validated`

            - input_type: EmbedInputType.

            - name: typing.Optional[str]. The name of the embed job.

            - truncate: typing.Optional[CreateEmbedJobRequestTruncate]. One of `START|END` to specify how the API will handle inputs longer than the maximum token length.

                                                                        Passing `START` will discard the start of the input. `END` will discard the end of the input. In both cases, input is discarded until the remaining input is exactly the maximum input token length for the model.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere import EmbedInputType
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.embed_jobs.create(
            model="model",
            dataset_id="dataset_id",
            input_type=EmbedInputType.SEARCH_DOCUMENT,
        )
        """
        _request: typing.Dict[str, typing.Any] = {"model": model, "dataset_id": dataset_id, "input_type": input_type}
        if name is not OMIT:
            _request["name"] = name
        if truncate is not OMIT:
            _request["truncate"] = truncate.value if truncate is not None else None
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "embed-jobs"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmbedJobResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmbedJob:
        """
        This API retrieves the details about an embed job started by the same user.

        Parameters:
            - id: str. The ID of the embed job to retrieve.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.embed_jobs.get(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"embed-jobs/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmbedJob, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        This API allows users to cancel an active embed job. Once invoked, the embedding process will be terminated, and users will be charged for the embeddings processed up to the cancellation point. It's important to note that partial results will not be available to users after cancellation.

        Parameters:
            - id: str. The ID of the embed job to cancel.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from cohere.base_client import AsyncBaseCohere

        client = AsyncBaseCohere(
            client_name="YOUR_CLIENT_NAME",
            token="YOUR_TOKEN",
        )
        await client.embed_jobs.cancel(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"embed-jobs/{jsonable_encoder(id)}/cancel"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class DatasetType(str, enum.Enum):
    """
    The type of the dataset
    """

    EMBED_INPUT = "embed-input"
    EMBED_RESULT = "embed-result"
    CLUSTER_RESULT = "cluster-result"
    CLUSTER_OUTLIERS = "cluster-outliers"
    RERANKER_FINETUNE_INPUT = "reranker-finetune-input"
    PROMPT_COMPLETION_FINETUNE_INPUT = "prompt-completion-finetune-input"
    SINGLE_LABEL_CLASSIFICATION_FINETUNE_INPUT = "single-label-classification-finetune-input"
    CHAT_FINETUNE_INPUT = "chat-finetune-input"
    MULTI_LABEL_CLASSIFICATION_FINETUNE_INPUT = "multi-label-classification-finetune-input"

    def visit(
        self,
        embed_input: typing.Callable[[], T_Result],
        embed_result: typing.Callable[[], T_Result],
        cluster_result: typing.Callable[[], T_Result],
        cluster_outliers: typing.Callable[[], T_Result],
        reranker_finetune_input: typing.Callable[[], T_Result],
        prompt_completion_finetune_input: typing.Callable[[], T_Result],
        single_label_classification_finetune_input: typing.Callable[[], T_Result],
        chat_finetune_input: typing.Callable[[], T_Result],
        multi_label_classification_finetune_input: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is DatasetType.EMBED_INPUT:
            return embed_input()
        if self is DatasetType.EMBED_RESULT:
            return embed_result()
        if self is DatasetType.CLUSTER_RESULT:
            return cluster_result()
        if self is DatasetType.CLUSTER_OUTLIERS:
            return cluster_outliers()
        if self is DatasetType.RERANKER_FINETUNE_INPUT:
            return reranker_finetune_input()
        if self is DatasetType.PROMPT_COMPLETION_FINETUNE_INPUT:
            return prompt_completion_finetune_input()
        if self is DatasetType.SINGLE_LABEL_CLASSIFICATION_FINETUNE_INPUT:
            return single_label_classification_finetune_input()
        if self is DatasetType.CHAT_FINETUNE_INPUT:
            return chat_finetune_input()
        if self is DatasetType.MULTI_LABEL_CLASSIFICATION_FINETUNE_INPUT:
            return multi_label_classification_finetune_input()



import typing

import httpx

from .environment import BaseCohereEnvironment
from .base_client import BaseCohere, AsyncBaseCohere


class Client(BaseCohere):
    def __init__(
            self,
            api_key: typing.Union[str, typing.Callable[[], str]],
            *,
            base_url: typing.Optional[str] = None,
            environment: BaseCohereEnvironment = BaseCohereEnvironment.PRODUCTION,
            client_name: typing.Optional[str] = None,
            timeout: typing.Optional[float] = 60,
            httpx_client: typing.Optional[httpx.Client] = None,
    ):
        BaseCohere.__init__(
            self,
            base_url=base_url,
            environment=environment,
            client_name=client_name,
            token=api_key,
            timeout=timeout,
            httpx_client=httpx_client,
        )


class AsyncClient(AsyncBaseCohere):
    def __init__(
            self,
            api_key: typing.Union[str, typing.Callable[[], str]],
            *,
            base_url: typing.Optional[str] = None,
            environment: BaseCohereEnvironment = BaseCohereEnvironment.PRODUCTION,
            client_name: typing.Optional[str] = None,
            timeout: typing.Optional[float] = 60,
            httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        AsyncBaseCohere.__init__(
            self,
            base_url=base_url,
            environment=environment,
            client_name=client_name,
            token=api_key,
            timeout=timeout,
            httpx_client=httpx_client,
        )



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CompatibleEndpoint(str, enum.Enum):
    """
    One of the Cohere API endpoints that the model can be used with.
    """

    CHAT = "chat"
    EMBED = "embed"
    CLASSIFY = "classify"
    SUMMARIZE = "summarize"
    RERANK = "rerank"
    RATE = "rate"
    GENERATE = "generate"

    def visit(
        self,
        chat: typing.Callable[[], T_Result],
        embed: typing.Callable[[], T_Result],
        classify: typing.Callable[[], T_Result],
        summarize: typing.Callable[[], T_Result],
        rerank: typing.Callable[[], T_Result],
        rate: typing.Callable[[], T_Result],
        generate: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is CompatibleEndpoint.CHAT:
            return chat()
        if self is CompatibleEndpoint.EMBED:
            return embed()
        if self is CompatibleEndpoint.CLASSIFY:
            return classify()
        if self is CompatibleEndpoint.SUMMARIZE:
            return summarize()
        if self is CompatibleEndpoint.RERANK:
            return rerank()
        if self is CompatibleEndpoint.RATE:
            return rate()
        if self is CompatibleEndpoint.GENERATE:
            return generate()
